/**************************************************************************************

Copyright (c) Hilscher Gesellschaft fuer Systemautomation mbH. All Rights Reserved.

***************************************************************************************

  $Id: ResetDlg.cpp 14388 2022-01-14 13:36:55Z LContreras $:

  Description:
    Implementation of the "Device/Reset" dialog

  Changes:
    Date        Description
    -----------------------------------------------------------------------------------
    2021-08-19  Added reset mode selection depending on netX chip type
    2021-08-18  Limit input of edit control for reset parameter to two characters
    2021-08-12  Restructure Reset dialog
                - offer separate GUI elements
                - elements only visible for system channel
                - not active if Reset Mode has not defined it
    2018-11-15  Reset option "Update Start" added
    2011-09-13  Reset option "Bootstart" added
    2007-03-23  COLDSTART/WARMSTART selection included
    2006-07-05  initial version

**************************************************************************************/

///////////////////////////////////////////////////////////////////////////////////////////
/// \file ResetDlg.cpp
///  Implementation of the "Device/Reset" dialog
///////////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "cifXTest.h"
#include "ResetDlg.h"
#include "CifXTestDlg.h"

#include "Hil_DualPortMemory.h" /* Reset modes */

static struct CHANNEL_RESET_MODEStag
{
  LPCTSTR       szMode;
  unsigned long ulMode;
} s_atChannelResetModes[] =
{
  {_T("System Start"), CIFX_RESETEX_SYSTEMSTART},
  {_T("Channel Init"), CIFX_CHANNELINIT},
};

typedef struct SYSTEM_RESET_MODEStag
{
  LPCTSTR       szMode;
  unsigned long ulMode;
} SYSTEM_RESET_MODE;

SYSTEM_RESET_MODE s_atSystemResetModes1[] =
{
  {_T("System Start"), CIFX_RESETEX_SYSTEMSTART},
  {_T("Boot Start"),   CIFX_RESETEX_BOOTSTART},
  {NULL, 0},
};

SYSTEM_RESET_MODE s_atSystemResetModes2[] =
{
  {_T("System Start"), CIFX_RESETEX_SYSTEMSTART},
  {_T("Boot Start"),   CIFX_RESETEX_BOOTSTART},
  {_T("Update Start"), CIFX_RESETEX_UPDATESTART},
  {NULL, 0},
};

IMPLEMENT_DYNAMIC(CResetDlg, CBaseDialog)

/////////////////////////////////////////////////////////////////////////////
/// Default Constructor
///   \param pParent Parent Window
/////////////////////////////////////////////////////////////////////////////
CResetDlg::CResetDlg(CWnd* pParent /*=NULL*/)
	: CBaseDialog(CResetDlg::IDD, pParent, false)
  , m_ulTimeout(CIFX_TO_FIRMWARE_UPDATE)
  , m_ulResetParam(0)
  , m_fResetExDeleteRemanent(false)
  , m_fResetExVerify(false)
{
}

/////////////////////////////////////////////////////////////////////////////
/// Destructor
/////////////////////////////////////////////////////////////////////////////
CResetDlg::~CResetDlg()
{
}

/////////////////////////////////////////////////////////////////////////////
/// DDX/DDV support
///   \param pDX
/////////////////////////////////////////////////////////////////////////////
void CResetDlg::DoDataExchange(CDataExchange* pDX)
{
  CBaseDialog::DoDataExchange(pDX);
  DDX_Control(pDX, IDC_CB_RESET, m_cResetModeCtrl);
  DDX_Control(pDX, IDC_CHK_RESETEX_DELETE_REMANENT, m_cResetExDeleteRemanentCtrl);
  DDX_Control(pDX, IDC_CHK_RESETEX_VERIFY, m_cResetExVerifyCtrl);
}


BEGIN_MESSAGE_MAP(CResetDlg, CBaseDialog)
  ON_CBN_SELCHANGE(IDC_CB_RESET, OnCbnSelchangeCbReset)
  ON_EN_KILLFOCUS(IDC_EDT_TIMEOUT, OnEnKillfocusEdtTimeout)
  ON_EN_KILLFOCUS(IDC_EDT_RESETPARAM, OnEnKillfocusEdtParam)
  ON_BN_CLICKED(IDC_BTN_RESET, OnBnClickedBtnReset)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
/// First Time dialog initialization. Fill Combobox control with reset modes
///   \return TRUE
/////////////////////////////////////////////////////////////////////////////
BOOL CResetDlg::OnInitDialog()
{
  CBaseDialog::OnInitDialog();

  SetDlgItemInt(IDC_EDT_TIMEOUT, m_ulTimeout, FALSE);
  SetDlgItemInt(IDC_EDT_RESETPARAM, m_ulResetParam, FALSE);

  /* Limit number of characters for reset parameter.
  *  Valid range for reset parameter is 0..15 (one byte) -> two digits needed in decimal representation. */
  GetDlgItem(IDC_EDT_RESETPARAM)->SendMessage(EM_SETLIMITTEXT, (WPARAM) 2, 0);

  return TRUE;  // return TRUE unless you set the focus to a control
  // EXCEPTION: OCX Property Pages should return FALSE
}

/////////////////////////////////////////////////////////////////////////////
/// Reset Mode selection changed event
/////////////////////////////////////////////////////////////////////////////
void CResetDlg::OnCbnSelchangeCbReset()
{
  int iCurSel = m_cResetModeCtrl.GetCurSel();
  ASSERT(-1 != iCurSel);

  m_ulResetMode = (unsigned long)m_cResetModeCtrl.GetItemData(iCurSel);

  if(CcifXTestDlg::s_pcDevice)
  {
    bool fUpdateStart = false;

    if(CcifXTestDlg::s_pcDevice->IsNetX90or4000())
    {
      fUpdateStart = true;
    }

    if(CcifXTestDlg::s_pcDevice->IsSystemDevice())
    {
      /* Evaluate which options are allowed for given Reset mode
         - Verify installable firmware
         - Delete Remanent
         - Reset Parameter
         Elements are greyed out if unavailable and the accompanying state/value is reset (default).
      */

      /* Verify firmware checkbox */
      if(CIFX_RESETEX_UPDATESTART == m_ulResetMode)
        GetDlgItem(IDC_CHK_RESETEX_VERIFY)->EnableWindow(SW_SHOW);
      else
      {
        GetDlgItem(IDC_CHK_RESETEX_VERIFY)->EnableWindow(SW_HIDE);
        m_cResetExVerifyCtrl.PostMessage(BM_SETCHECK, BST_UNCHECKED, 0);
      }

      /* Delete remanent checkbox and border for resetex */
      if( (CIFX_RESETEX_BOOTSTART == m_ulResetMode) ||
          (CIFX_RESETEX_UPDATESTART == m_ulResetMode) )
      {
        GetDlgItem(IDC_STATIC_GROUP_RESETEX)->EnableWindow(SW_SHOW);
        GetDlgItem(IDC_CHK_RESETEX_DELETE_REMANENT)->EnableWindow(SW_SHOW);
      }
      else
      {
        GetDlgItem(IDC_STATIC_GROUP_RESETEX)->EnableWindow(SW_HIDE);
        GetDlgItem(IDC_CHK_RESETEX_DELETE_REMANENT)->EnableWindow(SW_HIDE);
        m_cResetExDeleteRemanentCtrl.PostMessage(BM_SETCHECK, BST_UNCHECKED, 0);
      }

      /* Reset parameter (edit box) */
      if(CIFX_RESETEX_UPDATESTART == m_ulResetMode)
      {
        GetDlgItem(IDC_EDT_RESETPARAM)->EnableWindow(SW_SHOW);
        GetDlgItem(IDC_STATIC_RESETPARAM)->EnableWindow(SW_SHOW);
      }
      else
      {
        GetDlgItem(IDC_EDT_RESETPARAM)->EnableWindow(SW_HIDE);
        GetDlgItem(IDC_STATIC_RESETPARAM)->EnableWindow(SW_HIDE);
        SetDlgItemInt(IDC_EDT_RESETPARAM, 0, FALSE);
     }
    }
  }

}

/////////////////////////////////////////////////////////////////////////////
/// Timeout editing end event
/////////////////////////////////////////////////////////////////////////////
void CResetDlg::OnEnKillfocusEdtTimeout()
{
  CString csTemp;
  GetDlgItemText(IDC_EDT_TIMEOUT, csTemp);

  TCHAR* pszEnd = NULL;
  m_ulTimeout = _tcstoul(csTemp, &pszEnd, 10);

  csTemp.Format(_T("%u"), m_ulTimeout);
  SetDlgItemText(IDC_EDT_TIMEOUT, csTemp);
}

/////////////////////////////////////////////////////////////////////////////
/// Reset parameter editing end event
/////////////////////////////////////////////////////////////////////////////
void CResetDlg::OnEnKillfocusEdtParam()
{
  CString csTemp;
  GetDlgItemText(IDC_EDT_RESETPARAM, csTemp);

  TCHAR* pszEnd = NULL;
  m_ulResetParam = _tcstoul(csTemp, &pszEnd, 10);

  /* Limit value to allowed range (0..15) */
  if((HIL_SYS_CONTROL_RESET_PARAM_MASK >> HIL_SYS_CONTROL_RESET_PARAM_SRT) < m_ulResetParam)
    m_ulResetParam = (HIL_SYS_CONTROL_RESET_PARAM_MASK >> HIL_SYS_CONTROL_RESET_PARAM_SRT);

  csTemp.Format(_T("%u"), m_ulResetParam);
  SetDlgItemText(IDC_EDT_RESETPARAM, csTemp);
}

/////////////////////////////////////////////////////////////////////////////
/// Reset Button clicked event
/////////////////////////////////////////////////////////////////////////////
void CResetDlg::OnBnClickedBtnReset()
{
  if(CcifXTestDlg::s_pcDevice)
  {
    CWaitCursor cWait;
    uint32_t ulResetParam = 0;

    if(CcifXTestDlg::s_pcDevice->IsSystemDevice())
    {
      /* Evaluate all active elements to construct single reset parameter */
      if(GetDlgItem(IDC_CHK_RESETEX_DELETE_REMANENT)->IsWindowEnabled())
      {
        m_fResetExDeleteRemanent = m_cResetExDeleteRemanentCtrl.GetCheck() == BST_CHECKED;
        if(m_fResetExDeleteRemanent)
          /* Set delete remanent bit into parameter */
          ulResetParam |= 1 << HIL_SYS_CONTROL_RESET_FLAG_CLEAR_REMANENT_SRT;
      }

      if(GetDlgItem(IDC_CHK_RESETEX_VERIFY)->IsWindowEnabled())
      {
        m_fResetExVerify = m_cResetExVerifyCtrl.GetCheck() == BST_CHECKED;
        if(m_fResetExVerify)
          /* Set verify bit into parameter */
          ulResetParam |= 1 << HIL_SYS_CONTROL_RESET_FLAG_VERIFY_INSTALL_SRT;
      }

      if(GetDlgItem(IDC_EDT_RESETPARAM)->IsWindowEnabled())
      {
        /* Insert reset parameter to correct position */
        ulResetParam |= m_ulResetParam << HIL_SYS_CONTROL_RESET_PARAM_SRT;
      }
    }

    long lRet = CcifXTestDlg::s_pcDevice->Reset(m_ulResetMode, m_ulTimeout, ulResetParam);

    CString csTemp;
    if(CIFX_NO_ERROR != lRet)
    {
      csTemp.Format(_T("0x%08X\r\n%s"),
                    lRet,
                    (LPCTSTR)CcifXTestDlg::s_pcDevice->GetErrorDescription(lRet));

    }
    SetDlgItemText(IDC_LASTERR, csTemp);
  }
}

/////////////////////////////////////////////////////////////////////////////
/// Cyclic thread function (unused)
/////////////////////////////////////////////////////////////////////////////
void CResetDlg::ThreadFunction()
{
}

/////////////////////////////////////////////////////////////////////////////
/// Device Change notification (unused)
///   \param pcDevice NULL if device was closed, otherwise device has changed
/////////////////////////////////////////////////////////////////////////////
void CResetDlg::OnUpdateDevice(CCifXDeviceBase* pcDevice)
{
  bool fSysDevice   = false;

  SYSTEM_RESET_MODE* ptResetMode = &s_atSystemResetModes1[0];

  if (NULL != pcDevice)
  {
    if (pcDevice->IsSystemDevice())
      fSysDevice = true;

    if (pcDevice->IsNetX90or4000())
    {
      ptResetMode = &s_atSystemResetModes2[0];
    }
  }

  m_cResetModeCtrl.ResetContent();

  if (fSysDevice)
  {
    /* System channel modes */
    for(int iIdx = 0; NULL != ptResetMode->szMode; ++iIdx)
    {
      int iItem = m_cResetModeCtrl.AddString(ptResetMode->szMode);
      m_cResetModeCtrl.SetItemData(iItem, ptResetMode->ulMode);
      ptResetMode++;
    }

    /* Reset on System channel allows additional reset modes such as Bootstart and Updatestart via (ResetEx) */
    GetDlgItem(IDC_STATIC_GROUP_RESETEX)->ShowWindow(SW_SHOW);
    GetDlgItem(IDC_STATIC_RESETPARAM)->ShowWindow(SW_SHOW);
    GetDlgItem(IDC_EDT_RESETPARAM)->ShowWindow(SW_SHOW);
    GetDlgItem(IDC_CHK_RESETEX_VERIFY)->ShowWindow(SW_SHOW);
    GetDlgItem(IDC_CHK_RESETEX_DELETE_REMANENT)->ShowWindow(SW_SHOW);

  } else
  {
    /* Communication channel modes */
    for(int iIdx = 0; iIdx < sizeof(s_atChannelResetModes) / sizeof(s_atChannelResetModes[0]); ++iIdx)
    {
      int iItem = m_cResetModeCtrl.AddString(s_atChannelResetModes[iIdx].szMode);
      m_cResetModeCtrl.SetItemData(iItem, s_atChannelResetModes[iIdx].ulMode);
    }

    /* No extended reset mode handling supported */
    GetDlgItem(IDC_STATIC_GROUP_RESETEX)->ShowWindow(SW_HIDE);
    GetDlgItem(IDC_STATIC_RESETPARAM)->ShowWindow(SW_HIDE);
    GetDlgItem(IDC_EDT_RESETPARAM)->ShowWindow(SW_HIDE);
    GetDlgItem(IDC_CHK_RESETEX_VERIFY)->ShowWindow(SW_HIDE);
    GetDlgItem(IDC_CHK_RESETEX_DELETE_REMANENT)->ShowWindow(SW_HIDE);
  }

  /* Select first entry */
  m_cResetModeCtrl.SetCurSel(0);
  OnCbnSelchangeCbReset();
}
